/***************************************************************************
 *   Copyright (C) 2008, 2009 by Miguel Escudero Gestal
 *   miguelescudero@users.sourceforge.net
 *   
 *   This file is part of QChartDiary
 *
 *   QChartDiary is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   QChartDiary is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program. If not, see <http://www.gnu.org/licenses/>.
 ***************************************************************************/

#include "chartwindowimpl.h"

#include <QComboBox>
#include <QDateEdit>
#include <QGridLayout>
#include <QGroupBox>
#include <QMessageBox>
#include <QSettings>
#include <QSqlQuery>


//PUBLIC


ChartWindowImpl::ChartWindowImpl(QWidget * parent, Qt::WFlags f)
	: QMainWindow(parent, f)
{
	setupUi(this);
	
	createObjects();
	createTreeWidgetItems();
	sendActiveColumns();
	
	dateEditFinalDate->setDate(QDate::currentDate());
	readSettings();
}


//PUBLIC SLOTS


void ChartWindowImpl::activateAllUserColumns(QTreeWidgetItem *treeWidgetItem, int column)
{
	for(int i = 0; i < treeWidgetItems.size(); i++)
	{
		//Comprobamos solo los widgets que contienen el nombre de usuario
		if(treeWidgetItem->text(column) == userNames[i])
		{
			//Marcamos o desmarcamos todos los hijos del widget
			for(int j = 0; j < treeWidgetItems[i]->childCount(); j++)
			{
				treeWidgetItems[i]->child(j)->setCheckState(column, treeWidgetItems[i]->checkState(column));
			}
		}
	}
}

void ChartWindowImpl::sendActiveColumns()
{
	//Creamos un vector que contendrá el número de columnas activas que tiene cada usuario
	QVector <int> activeColumnsPerUser;
	QVector <int> activeColumns;
	QVector <QColor> activeColors;
	int columns;
	for(int i = 0; i < treeWidgetItems.size(); i++)
	{
		//Reiniciamos el contador de columnas del usuario actual
		columns = 0;
		
		for(int j = 0; j < treeWidgetItems[i]->childCount(); j++)
		{
			if(treeWidgetItems[i]->child(j)->checkState(0) == 2)
			{
				//Incrementamos el contador de columnas del usuario actual
				columns ++;
				
				activeColumns.append(j);
				activeColors.append(treeWidgetItems[i]->child(j)->foreground(0).color());
			}
		}
		activeColumnsPerUser.append(columns);
	}
	
	//Enviamos los datos a la gráfica y le mandamos dibujarla
	chart->setActiveColumns(activeColumns);
	chart->setActiveColumnsPerUser(activeColumnsPerUser);
	chart->setActiveColors(activeColors);
}

void ChartWindowImpl::setComboTempoItems(int num)
{
	QStringList items;
	switch(num)
	{
	case 0:
		comboTempo->clear();
		items<<tr("Daily");
		comboTempo->insertItems(0, items);
	break;
	case 1:
		comboTempo->clear();
		items<<tr("Daily")<<tr("Weekly")<<tr("Weelkly (average)")<<tr("Monthly")<<tr("Monthly (average)");
		items<<tr("Yearly")<<tr("Yearly (average)");
		comboTempo->insertItems(0, items);
	break;
	}
}

void ChartWindowImpl::setFinalDate(const QDate &date)
{
	//Controlamos que la fecha final sea mayor que la inicial
	if(date.daysTo(dateEditInitialDate->date()) > -1)
		dateEditFinalDate->setDate(dateEditInitialDate->date().addDays(1));
	chart->setFinalDate(date);
}

void ChartWindowImpl::setInitialDate(const QDate &date)
{
	//Controlamos que la fecha inicial sea menor que la final
	if(date.daysTo(dateEditFinalDate->date()) < 1)
		dateEditInitialDate->setDate(dateEditFinalDate->date().addDays(-1));
	chart->setInitialDate(date);
}

void ChartWindowImpl::writeSettings()
{
	QSettings settings;
	
	settings.beginGroup("ChartWindow");
	//settings.setValue("geometry", saveGeometry());
	//settings.setValue("size", size());
	//settings.setValue("pos", pos());
	settings.setValue("geometry", geometry());
	settings.setValue("initialDate", dateEditInitialDate->date());
	settings.setValue("comboChartType", comboChartType->currentIndex());
	settings.setValue("comboTempo", comboTempo->currentIndex());
	settings.endGroup();
}


//PROTECTED


void ChartWindowImpl::paintEvent(QPaintEvent */*event*/)
{
	//chart->drawGraphics();
}


//PRIVATE


void ChartWindowImpl::createObjects()
{
	dateEditInitialDate = new DateEdit(this);
	dateEditFinalDate = new DateEdit(this);
	dateEditInitialDate->setDisplayFormat("yyyy-MM-dd");
	dateEditFinalDate->setDisplayFormat("yyyy-MM-dd");
	dateEditInitialDate->setAlignment(Qt::AlignRight);
	dateEditFinalDate->setAlignment(Qt::AlignRight);
	horizontalLayoutInitialDate->addWidget(dateEditInitialDate);
	horizontalLayoutFinalDate->addWidget(dateEditFinalDate);
	
	chart = new Chart(this);
	scrollAreaChart->setWidget(chart);
	
	connect(dateEditInitialDate, SIGNAL(dateChanged(const QDate &)), this, SLOT(setInitialDate(const QDate &)));
	connect(dateEditFinalDate, SIGNAL(dateChanged(const QDate &)), this, SLOT(setFinalDate(const QDate &)));
	connect(comboChartType, SIGNAL(currentIndexChanged(int)), this, SLOT(setComboTempoItems(int)));
	connect(comboChartType, SIGNAL(currentIndexChanged(int)), chart, SLOT(setChartType(int)));
	connect(comboTempo, SIGNAL(currentIndexChanged(int)), chart, SLOT(setTempo(int)));
	//connect(this, SIGNAL(userChanged(const QString &)), chart, SLOT(setUser(const QString &)));
}

void ChartWindowImpl::createTreeWidgetItems()
{
	//Creamos los items padres, que contendrán cada nombre de usuario existente en la BD
	QSqlQuery userNamesQuery;
	userNamesQuery.exec("SELECT name, sql FROM sqlite_master WHERE type='table'"
		"AND name LIKE 'Diary%' ORDER BY name;");
	for(int i = 0; userNamesQuery.next(); i++)
	{
		userNames += userNamesQuery.value(0).toString().mid(5, userNamesQuery.value(0).toString().length());
		treeWidgetItems.append(new QTreeWidgetItem((QTreeWidget *)0, QStringList(userNames[i])));
		treeWidgetItems[i]->setCheckState(0, Qt::Unchecked);
		treeWidgetItems[i]->setFlags(Qt::ItemIsUserCheckable | Qt::ItemIsEnabled);
	}
	treeWidget->insertTopLevelItems(0, treeWidgetItems);
	
	//Enviamos al widget de la gráfica los nombres de los usuarios
	chart->setUserNames(userNames);
	
	//Expandimos los items añadidos
	for(int i = 0; i < treeWidgetItems.size(); i++)
		treeWidgetItems[i]->setExpanded(true);
	
	//Creamos y coloreamos los items hijos, es decir, los campos disponibles de cada usuario
	int color = 6;
	QSqlQuery query;
	for(int i = 0; i < treeWidgetItems.size(); i++)
	{
		query.exec("SELECT Id, FullName FROM Fields" + userNames[i]);
		while(query.next())
		{
			treeWidgetItems[i]->insertChild(query.value(0).toInt() - 1,
				(new QTreeWidgetItem(treeWidgetItems[i], QStringList(query.value(1).toString()))));
			treeWidgetItems[i]->child(query.value(0).toInt() - 1)->setCheckState(0, Qt::Unchecked);
			treeWidgetItems[i]->child(query.value(0).toInt() - 1)->setFlags(Qt::ItemIsUserCheckable
				| Qt::ItemIsEnabled);
			
			if(color < 14)
				color++;
			else
				color = 7;
			treeWidgetItems[i]->child(query.value(0).toInt() - 1)->setForeground(0, QBrush(Qt::GlobalColor(color)));
		}
	}
	
	connect(treeWidget, SIGNAL(itemClicked(QTreeWidgetItem *, int)),
		this, SLOT(activateAllUserColumns(QTreeWidgetItem *, int)));
	connect(treeWidget, SIGNAL(itemClicked(QTreeWidgetItem *, int)),
		this, SLOT(sendActiveColumns()));
}

void ChartWindowImpl::readSettings()
{
	QSettings settings;
	
	settings.beginGroup("ChartWindow");
	//restoreGeometry(settings.value("geometry").toByteArray());
	//resize(settings.value("size", QSize(900, 480)).toSize());
	//move(settings.value("pos", QPoint(200, 200)).toPoint());
	setGeometry(settings.value("geometry", QRect(100, 100, 900, 480)).toRect());
	dateEditInitialDate->setDate(settings.value("initialDate", QDate(2008, 4, 12)).toDate());
	comboChartType->setCurrentIndex(settings.value("comboChartType", int(1)).toInt());
	comboTempo->setCurrentIndex(settings.value("comboTempo", int(0)).toInt());
	settings.endGroup();
}
